@@ -48,7 +48,7 @@ static GtkHTMLClassProperties *actual_prop;
 /* static GList *saved_bindings;
    static GList *orig_bindings; */
 
-static gchar *home_rcfile;
+/* static gchar *home_rcfile; */
 
 static void
 set_ui ()
@@ -60,7 +60,7 @@ set_ui ()
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (anim_check), actual_prop->animations);
 
 #define SET_FONT(f,w) \
-	gnome_font_picker_set_font_name (GNOME_FONT_PICKER (w), actual_prop-> ## f);
+	gnome_font_picker_set_font_name (GNOME_FONT_PICKER (w), actual_prop-> f);
 
 	SET_FONT (font_var,       variable);
 	SET_FONT (font_fix,       fixed);
@@ -118,10 +118,10 @@ apply_fonts ()
 	actual_prop->animations = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (anim_check));
 
 #define APPLY(f,s,w) \
-	g_free (actual_prop-> ## f); \
-	actual_prop-> ## f = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (w))); \
+	g_free (actual_prop-> f); \
+	actual_prop-> f = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (w))); \
 	size_str = get_attr (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (w)), 7); \
-	actual_prop-> ## s = atoi (size_str); \
+	actual_prop-> s = atoi (size_str); \
 	g_free (size_str)
 
 	APPLY (font_var,       font_var_size,       variable);
@@ -144,7 +144,9 @@ apply_editable (void)
 	/* properties */
 	actual_prop->magic_links = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (magic_check));
 	keymap_name = gnome_bindings_properties_get_keymap_name (GNOME_BINDINGS_PROPERTIES (bi));
-	if (!strcmp (keymap_name, EMACS_KEYMAP_NAME)) {
+	if (!keymap_name) {
+		keymap_id = "ms";
+	} else if (!strcmp (keymap_name, EMACS_KEYMAP_NAME)) {
 		keymap_id = "emacs";
 	} else if (!strcmp (keymap_name, XEMACS_KEYMAP_NAME)) {
 		keymap_id = "xemacs";
@@ -221,12 +223,12 @@ static void
 picker_clicked (GtkWidget *w, gpointer data)
 {
 	gchar *mono_spaced [] = { "c", "m", NULL };
-	gchar *proportional [] = { "p", NULL };
 	
-	gtk_font_selection_dialog_set_filter (SELECTOR (w),
-					      GTK_FONT_FILTER_BASE, GTK_FONT_ALL,
-					      NULL, NULL, NULL, NULL,
-					      GPOINTER_TO_INT (data) ? proportional : mono_spaced, NULL);
+	if (!GPOINTER_TO_INT (data))
+		gtk_font_selection_dialog_set_filter (SELECTOR (w),
+						      GTK_FONT_FILTER_BASE, GTK_FONT_ALL,
+						      NULL, NULL, NULL, NULL,
+						      mono_spaced, NULL);
 }
 
 static void
@@ -272,9 +274,9 @@ setup (void)
         g_free (base); \
 	g_free (rcfile)
 	
-	home_rcfile = g_strconcat (gnome_util_user_home (), "/.gnome/gtkhtml-bindings-custom", NULL);
-	gtk_rc_parse (home_rcfile);
-	g_free (home_rcfile);
+	/* home_rcfile = g_strconcat (gnome_util_user_home (), "/.gnome/gtkhtml-bindings-custom", NULL);
+	   gtk_rc_parse (home_rcfile); */
+
 	LOAD ("emacs");
 	LOAD ("xemacs");
 	LOAD ("ms");
@@ -353,7 +355,7 @@ main (int argc, char **argv)
 
         capplet_gtk_main ();
 
-	g_free (home_rcfile);
+	/* g_free (home_rcfile); */
 
         return 0;
 }
