@@ -28,6 +28,8 @@
 #include "htmlcolorset.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
+#include "htmlengine-edit.h"
+#include "htmlengine-edit-table.h"
 #include "htmlengine-edit-tablecell.h"
 #include "htmlengine-save.h"
 #include "htmlimage.h"
@@ -36,10 +38,17 @@
 
 #include "config.h"
 #include "properties.h"
-#include "dialog.h"
 #include "cell.h"
 #include "utils.h"
 
+typedef enum
+{
+	CELL,
+	ROW,
+	COLUMN,
+	TABLE
+} CellScope;
+
 typedef struct
 {	
 	GtkHTMLControlData *cd;
@@ -91,6 +100,9 @@ typedef struct
 	gboolean   changed_heading;
 	GtkWidget *option_heading;
 
+	CellScope  scope;
+	GtkWidget *option_scope;
+
 	gboolean   disable_change;
 
 } GtkHTMLEditCellProperties;
@@ -101,7 +113,9 @@ typedef struct
 static void
 fill_sample (GtkHTMLEditCellProperties *d)
 {
-	gchar *body, *html, *bg_color, *bg_pixmap, *valign, *halign, *width, *height, *wrap;
+	GString *str;
+	gchar *body, *bg_color, *bg_pixmap, *valign, *halign, *width, *height, *wrap, *cell;
+	gint r, c;
 
 	body      = html_engine_save_get_sample_body (d->cd->html->engine, NULL);
 
@@ -128,28 +142,41 @@ fill_sample (GtkHTMLEditCellProperties *d)
 		? g_strdup_printf (" height=\"%d%s\"", d->height, d->height_percent ? "%" : "") : g_strdup ("");
 	wrap    = d->wrap ? " nowrap" : "";
 
-	html    = g_strconcat (body,
-			       "<table border=1 cellpadding=4 cellspacing=2>"
-			       "<tr><td>&nbsp;Other&nbsp;<br>&nbsp;Other&nbsp;<br>"
-			       "&nbsp;Other&nbsp;<br>&nbsp;Other&nbsp;<br>",
-			       "&nbsp;Other&nbsp;<br>&nbsp;Other&nbsp;</td><",
-			       d->heading ? "th" : "td",
-			       bg_color, bg_pixmap, halign, valign, width, height, wrap,
-			       ">The quick brown fox jumps over the lazy dog. The quick brown fox jumps over the lazy dog."
-			       "</td><td>&nbsp;Other&nbsp;<br>&nbsp;Other&nbsp;<br>"
-			       "&nbsp;Other&nbsp;<br>&nbsp;Other&nbsp;<br>",
-			       "&nbsp;Other&nbsp;<br>&nbsp;Other&nbsp;</td></tr>"
-			       "<tr><td>&nbsp;Other&nbsp;</td><td>&nbsp;Other&nbsp;</td><td>&nbsp;Other&nbsp;</td></tr>"
-			       "</table>", NULL);
-	printf ("html: %s\n", html);
-	gtk_html_load_from_string (d->sample, html, -1);
+	cell    = g_strconcat ("<", d->heading ? "th" : "td",
+			       bg_color, bg_pixmap, halign, valign, width, height, wrap, ">", NULL);
+
+	str = g_string_new (body);
+	g_string_append (str, "<table border=1 cellpadding=4 cellspacing=2>");
+
+	for (r = 0; r < 2; r ++) {
+		g_string_append (str, "<tr>");
+		for (c = 0; c < 3; c ++) {
+			if ((r == 0 && c == 1)
+			    || (d->scope == ROW && r == 0)
+			    || (d->scope == COLUMN && c == 1)
+			    || d->scope == TABLE)
+				g_string_append (str, cell);
+			else
+				g_string_append (str, "<td>");
+
+			g_string_append (str, r == 0 && c == 1
+					 ? "The quick brown fox jumps over the lazy dog. "
+					 "The quick brown fox jumps over the lazy dog."
+					 : "&nbsp;Other&nbsp;");
+			g_string_append (str, "</td>");
+		}
+		g_string_append (str, "</tr>");
+	}
+	g_string_append (str, "</table>");
+	gtk_html_load_from_string (d->sample, str->str, -1);
 
 	g_free (halign);
 	g_free (valign);
 	g_free (bg_color);
 	g_free (bg_pixmap);
 	g_free (body);
-	g_free (html);
+	g_free (cell);
+	g_string_free (str, TRUE);
 }
 
 static GtkHTMLEditCellProperties *
@@ -332,13 +359,26 @@ changed_heading (GtkWidget *w, GtkHTMLEditCellProperties *d)
 	CHANGE;	
 }
 
+static void
+changed_scope (GtkWidget *w, GtkHTMLEditCellProperties *d)
+{
+	d->scope = g_list_index (GTK_MENU_SHELL (w)->children, gtk_menu_get_active (GTK_MENU (w)));
+	FILL;
+	CHANGE;	
+}
+
+/*
+ * FIX: set spin adjustment upper to 100000
+ *      as glade cannot set it now
+ */
+#define UPPER_FIX(x) gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (d->spin_ ## x))->upper = 100000.0
+
 static GtkWidget *
 cell_widget (GtkHTMLEditCellProperties *d)
 {
 	HTMLColor *color;
 	GtkWidget *cell_page;
 	GladeXML *xml;
-	gchar     *dir;
 
 	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "cell_page");
 	if (!xml)
@@ -360,11 +400,15 @@ cell_widget (GtkHTMLEditCellProperties *d)
 	d->check_bg_pixmap = glade_xml_get_widget (xml, "check_cell_bg_pixmap");
 	gtk_signal_connect (GTK_OBJECT (d->check_bg_pixmap), "toggled", set_has_bg_pixmap, d);
 	d->entry_bg_pixmap = glade_xml_get_widget (xml, "entry_cell_bg_pixmap");
+
+	d->disable_change = TRUE;
+	/* fix for broken gnome-libs, could be removed once gnome-libs are fixed */
+	gnome_entry_load_history (GNOME_ENTRY (gnome_pixmap_entry_gnome_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))));
+	our_gnome_pixmap_entry_set_last_dir (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap));
+	d->disable_change = FALSE;
+
 	gtk_signal_connect (GTK_OBJECT (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))),
 			    "changed", GTK_SIGNAL_FUNC (changed_bg_pixmap), d);
-	dir = getcwd (NULL, 0);
-	gnome_pixmap_entry_set_pixmap_subdir (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap), dir);
-	free (dir);
 
 	d->option_halign = glade_xml_get_widget (xml, "option_cell_halign");
 	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_halign))), "selection-done",
@@ -374,6 +418,7 @@ cell_widget (GtkHTMLEditCellProperties *d)
 			    changed_valign, d);
 
 	d->spin_width   = glade_xml_get_widget (xml, "spin_cell_width");
+	UPPER_FIX (width);
 	gtk_signal_connect (GTK_OBJECT (d->spin_width), "changed", changed_width, d);
 	d->check_width  = glade_xml_get_widget (xml, "check_cell_width");
 	gtk_signal_connect (GTK_OBJECT (d->check_width), "toggled", set_has_width, d);
@@ -382,6 +427,7 @@ cell_widget (GtkHTMLEditCellProperties *d)
 			    changed_width_percent, d);
 
 	d->spin_height   = glade_xml_get_widget (xml, "spin_cell_height");
+	UPPER_FIX (height);
 	gtk_signal_connect (GTK_OBJECT (d->spin_height), "changed", changed_height, d);
 	d->check_height  = glade_xml_get_widget (xml, "check_cell_height");
 	gtk_signal_connect (GTK_OBJECT (d->check_height), "toggled", set_has_height, d);
@@ -396,6 +442,10 @@ cell_widget (GtkHTMLEditCellProperties *d)
 	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_heading))), "selection-done",
 			    changed_heading, d);
 
+	d->option_scope   = glade_xml_get_widget (xml, "option_cell_scope");
+	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_scope))), "selection-done",
+			    changed_scope, d);
+
 	gtk_box_pack_start (GTK_BOX (cell_page), sample_frame (&d->sample), FALSE, FALSE, 0);
 
 	gtk_widget_show_all (cell_page);
@@ -506,64 +556,121 @@ cell_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	return rv;
 }
 
+static void
+cell_apply_1 (HTMLTableCell *cell, GtkHTMLEditCellProperties *d)
+{
+	if (d->changed_bg_color)
+		html_engine_table_cell_set_bg_color (d->cd->html->engine, cell, d->has_bg_color ? &d->bg_color : NULL);
+
+	if (d->changed_bg_pixmap) {
+		gchar *url = d->has_bg_pixmap ? g_strconcat ("file://", d->bg_pixmap, NULL) : NULL;
+
+		html_engine_table_cell_set_bg_pixmap (d->cd->html->engine, cell, url);
+		g_free (url);
+	}
+
+	if (d->changed_halign)
+		html_engine_table_cell_set_halign (d->cd->html->engine, cell, d->halign);
+
+	if (d->changed_valign)
+		html_engine_table_cell_set_valign (d->cd->html->engine, cell, d->valign);
+
+	if (d->changed_wrap)
+		html_engine_table_cell_set_no_wrap (d->cd->html->engine, cell, d->wrap);
+
+	if (d->changed_heading)
+		html_engine_table_cell_set_heading (d->cd->html->engine, cell, d->heading);
+
+	if (d->changed_width)
+		html_engine_table_cell_set_width (d->cd->html->engine, cell,
+						  d->has_width ? d->width : 0, d->has_width ? d->width_percent : FALSE);
+}
+
+static void
+cell_apply_row (GtkHTMLEditCellProperties *d)
+{
+	HTMLTableCell *cell;
+	HTMLEngine *e = d->cd->html->engine;
+
+	if (html_engine_table_goto_row (e, d->cell->row)) {
+		cell = html_engine_get_table_cell (e);
+
+		while (cell && cell->row == d->cell->row) {
+			if (HTML_OBJECT (cell)->parent == HTML_OBJECT (d->cell)->parent)
+				cell_apply_1 (cell, d);
+			html_engine_next_cell (e, FALSE);
+			cell = html_engine_get_table_cell (e);
+		}
+	}
+}
+
+static void
+cell_apply_col (GtkHTMLEditCellProperties *d)
+{
+	HTMLTableCell *cell;
+	HTMLEngine *e = d->cd->html->engine;
+
+	if (html_engine_table_goto_col (e, d->cell->col)) {
+		cell = html_engine_get_table_cell (e);
+
+		while (cell) {
+			if (cell->col == d->cell->col && HTML_OBJECT (cell)->parent == HTML_OBJECT (d->cell)->parent)
+				cell_apply_1 (cell, d);
+			html_engine_next_cell (e, FALSE);
+			cell = html_engine_get_table_cell (e);
+		}
+	}
+}
+
+static void
+cell_apply_table (GtkHTMLEditCellProperties *d)
+{
+	HTMLTableCell *cell;
+	HTMLEngine *e = d->cd->html->engine;
+
+	if (html_engine_table_goto_0_0 (e)) {
+		cell = html_engine_get_table_cell (e);
+
+		while (cell) {
+			if (HTML_OBJECT (cell)->parent == HTML_OBJECT (d->cell)->parent)
+				cell_apply_1 (cell, d);
+			html_engine_next_cell (e, FALSE);
+			cell = html_engine_get_table_cell (e);
+		}
+	}
+}
+
 void
 cell_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	GtkHTMLEditCellProperties *d = (GtkHTMLEditCellProperties *) get_data;
+	HTMLEngine *e = d->cd->html->engine;
+	gint position;
 
-	if (d->changed_bg_color) {
-		html_engine_table_cell_set_bg_color (d->cd->html->engine, d->cell, d->has_bg_color ? &d->bg_color : NULL);
-		d->changed_bg_color = FALSE;
-	}
-	if (d->changed_bg_pixmap) {
-		gchar *url = d->has_bg_pixmap ? g_strconcat ("file://", d->bg_pixmap, NULL) : NULL;
+	position = e->cursor->position;
 
-		html_engine_table_cell_set_bg_pixmap (d->cd->html->engine, d->cell, url);
-		g_free (url);
-		d->changed_bg_pixmap = FALSE;
+	switch (d->scope) {
+	case CELL:
+		cell_apply_1 (d->cell, d);
+		break;
+	case ROW:
+		cell_apply_row (d);
+		break;
+	case COLUMN:
+		cell_apply_col (d);
+		break;
+	case TABLE:
+		cell_apply_table (d);
 	}
-	if (d->changed_halign) {
-		html_engine_table_cell_set_halign (d->cd->html->engine, d->cell, d->halign);
-		d->changed_halign = FALSE;
-	}
-	if (d->changed_valign) {
-		html_engine_table_cell_set_valign (d->cd->html->engine, d->cell, d->valign);
-		d->changed_valign = FALSE;
-	}
-	if (d->changed_wrap) {
-		html_engine_table_cell_set_no_wrap (d->cd->html->engine, d->cell, d->wrap);
-		d->changed_wrap = FALSE;
-	}
-	if (d->changed_heading) {
-		html_engine_table_cell_set_heading (d->cd->html->engine, d->cell, d->heading);
-		d->changed_heading = FALSE;
-	}
-	if (d->changed_width) {
-		html_engine_table_cell_set_width (d->cd->html->engine, d->cell,
-						  d->has_width ? d->width : 0, d->has_width ? d->width_percent : FALSE);
-		d->changed_width = FALSE;
-	}
-	/* if (d->changed_spacing) {
-		html_engine_table_set_spacing (d->cd->html->engine, d->table, d->spacing);
-		d->changed_spacing = FALSE;
-	}
-	if (d->changed_padding) {
-		html_engine_table_set_padding (d->cd->html->engine, d->table, d->padding);
-		d->changed_padding = FALSE;
-	}
-	if (d->changed_border) {
-		html_engine_table_set_border_width (d->cd->html->engine, d->table, d->border, FALSE);
-		d->changed_border = FALSE;
-	}
-	if (d->changed_align) {
-		html_engine_table_set_align (d->cd->html->engine, d->table, d->align);
-		d->changed_align = FALSE;
-	}
-	if (d->changed_width) {
-		html_engine_table_set_width (d->cd->html->engine, d->table,
-					     d->has_width ? d->width : 0, d->has_width ? d->width_percent : FALSE);
-		d->changed_width = FALSE;
-		} */
+	html_cursor_jump_to_position (e->cursor, e, position);
+
+	d->changed_bg_color = FALSE;
+	d->changed_bg_pixmap = FALSE;
+	d->changed_halign = FALSE;
+	d->changed_valign = FALSE;
+	d->changed_wrap = FALSE;
+	d->changed_heading = FALSE;
+	d->changed_width = FALSE;
 }
 
 void
