@@ -160,7 +160,7 @@ image_widget (GtkHTMLControlData *cd, gpointer *set_data)
 {
 	GtkHTMLEditImageProperties *data = g_new0 (GtkHTMLEditImageProperties, 1);
 	GtkWidget *mhb, *hbox;
-	GtkWidget *vbox, *vb1;
+	GtkWidget *vb1;
 	GtkWidget *table;
 	GtkWidget *frame;
 	GtkWidget *menu;
@@ -228,7 +228,6 @@ image_widget (GtkHTMLControlData *cd, gpointer *set_data)
 	gtk_box_pack_start_defaults (GTK_BOX (vb1), frame);
 
 	/* spacing */
-	vbox = gtk_vbox_new (FALSE, 0);
 	frame = gtk_frame_new (_("Padding"));
 
 	table = gtk_table_new (2, 2, FALSE);
@@ -247,8 +246,6 @@ image_widget (GtkHTMLControlData *cd, gpointer *set_data)
 	gtk_box_pack_start (GTK_BOX (vb1), frame, FALSE, FALSE, 0);
 
 	/* size and spacing */
-	hbox = gtk_hbox_new (FALSE, 3);
-	vbox = gtk_vbox_new (FALSE, 0);
 	frame = gtk_frame_new (_("Size"));
 
 	table = gtk_table_new (4, 2, FALSE);
@@ -333,8 +330,8 @@ image_properties (GtkHTMLControlData *cd, gpointer *set_data)
 			    ip->url + off);
 
 #define SET_ADJ(which, nval, val) \
-        if (image->## val != nval) { \
-		gtk_adjustment_set_value (GTK_ADJUSTMENT (d->adj [which]), image->## val); \
+        if (image-> val != nval) { \
+		gtk_adjustment_set_value (GTK_ADJUSTMENT (d->adj [which]), image-> val); \
                 gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (d->check [which]), TRUE); \
 	} else { \
 		gtk_adjustment_set_value (GTK_ADJUSTMENT (d->adj [which]), 0); \
@@ -460,7 +457,7 @@ insert_or_apply (GtkHTMLControlData *cd, gpointer get_data, gboolean insert)
 		if (data->set [GTK_HTML_EDIT_IMAGE_BWIDTH])
 			html_image_set_border (image, border);
 		html_image_set_size     (image, width, height, width_percent, height_percent);
-		html_image_set_url      (image, file);
+		html_image_edit_set_url (image, file);
 		html_image_set_spacing  (image, hspace, vspace);
 		html_image_set_valign   (image, valign);
 	}
