@@ -218,7 +218,7 @@ indent_less_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cnam
 static void 
 spell_check_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 {
-	spell_check_document (cd);
+	spell_check_dialog (cd, TRUE);
 }
 
 BonoboUIVerb verbs [] = {
@@ -259,6 +259,7 @@ static struct {
 	{GTK_HTML_FONT_STYLE_SIZE_6, "/commands/FontSizeThree"},
 	{GTK_HTML_FONT_STYLE_SIZE_7, "/commands/FontSizeFour"},
 	{GTK_HTML_FONT_STYLE_BOLD,    "/commands/FormatBold"},
+	{GTK_HTML_FONT_STYLE_ITALIC, "/commands/FormatItalic"},
 	{GTK_HTML_FONT_STYLE_UNDERLINE, "/commands/FormatUnderline"},
 	{GTK_HTML_FONT_STYLE_STRIKEOUT, "/commands/FormatStrikeout"},
 	{GTK_HTML_FONT_STYLE_FIXED, "/commands/FormatFixed"},
@@ -461,13 +462,13 @@ menubar_update_paragraph_alignment (GtkHTML *html,
 	}
 }
 
-void
+static void
 menubar_update_paragraph_style (GtkHTML *html, 
 				GtkHTMLParagraphStyle style, 
 				GtkHTMLControlData *cd)
 {
 	BonoboUIComponent *uic;
-	char *path = NULL;
+	const char *path = NULL;
 	int i;
 
 	uic = bonobo_control_get_ui_component (cd->control);
@@ -529,6 +530,10 @@ menubar_update_format (GtkHTMLControlData *cd)
 				      "sensitive", sensitive, &ev);
 	bonobo_ui_component_set_prop (uic, "/commands/FormatUnderline",
 				      "sensitive", sensitive, &ev);
+	bonobo_ui_component_set_prop (uic, "/commands/FormatStrikeout",
+				      "sensitive", sensitive, &ev);
+	bonobo_ui_component_set_prop (uic, "/commands/FormatPlain",
+				      "sensitive", sensitive, &ev);
 	
 	bonobo_ui_component_set_prop (uic, "/commands/AlignLeft",
 				      "sensitive", sensitive, &ev);		
