@@ -51,6 +51,7 @@ struct _GtkHTMLPrivate {
 
 #ifdef GTKHTML_HAVE_GCONF
 	guint set_font_id;
+	guint notify_id;
 #endif
 #ifdef GTK_HTML_USE_XIM
 	GdkICAttr *ic_attr;
@@ -58,12 +59,15 @@ struct _GtkHTMLPrivate {
 #endif
 };
 
-void  gtk_html_private_calc_scrollbars  (GtkHTML                 *html);
+void  gtk_html_private_calc_scrollbars  (GtkHTML                 *html,
+					 gboolean                *changed_x,
+					 gboolean                *changed_y);
 void  gtk_html_editor_event_command     (GtkHTML                 *html,
 					 GtkHTMLCommandType       com_type);
 void  gtk_html_editor_event             (GtkHTML                 *html,
 					 GtkHTMLEditorEventType   event,
 					 GtkArg                 **args);
+void  gtk_html_api_set_language         (GtkHTML                 *html);
 
 #endif /* _GTKHTML_PRIVATE_H */
 
