@@ -20,10 +20,12 @@
 */
 
 #include <config.h>
-
 #include <stdio.h>
 #include <sys/types.h>
+#include <glib.h>
+#include <stdlib.h>
 
+#include "gtkhtml-compat.h"
 #include "gtkhtml-stream.h"
 
 
@@ -54,10 +56,19 @@ gtk_html_stream_destroy (GtkHTMLStream *stream)
 	g_free (stream);
 }
 
+/**
+ * gtk_html_stream_write:
+ * @stream:
+ * @buffer:
+ * @size:
+ *
+ * Write data to a GtkHTMLStream.
+ *
+ */
 void
 gtk_html_stream_write (GtkHTMLStream *stream,
 		       const gchar *buffer,
-		       guint size)
+		       size_t size)
 {
 	g_return_if_fail (stream != NULL);
 	g_return_if_fail (buffer != NULL);
@@ -67,6 +78,44 @@ gtk_html_stream_write (GtkHTMLStream *stream,
 		stream->write_func (stream, buffer, size, stream->user_data);
 }
 
+int
+gtk_html_stream_vprintf (GtkHTMLStream *stream,
+			 const char *format,
+			 va_list ap)
+{
+	size_t len = g_printf_string_upper_bound (format, ap);
+	char *buf = NULL;
+	char *mbuf = NULL;
+	int rv;
+
+	if (len < 8192)
+		buf = alloca (len);
+
+	if (buf == NULL)
+		buf = mbuf = g_malloc (len);
+		
+	rv = vsprintf (buf, format, ap);
+	gtk_html_stream_write (stream, buf, rv);
+
+	g_free (mbuf);
+	return rv;
+}
+
+int
+gtk_html_stream_printf (GtkHTMLStream *stream,
+			const char *format,
+			...)
+{
+	va_list ap;
+	int rv;
+
+	va_start (ap, format);
+	rv = gtk_html_stream_vprintf (stream, format, ap);
+	va_end (ap);
+
+	return rv;
+}
+
 void
 gtk_html_stream_close (GtkHTMLStream *stream,
 		       GtkHTMLStreamStatus status)
