@@ -75,7 +75,7 @@ calc_size (HTMLObject *clue,
 	changed = HTML_OBJECT_CLASS (&html_clue_class)->calc_size (clue, painter);
 
 	if (clue->parent != NULL)
-		lmargin = html_object_get_left_margin (clue->parent, clue->y);
+		lmargin = html_object_get_left_margin (clue->parent, painter, clue->y);
 
 	clue->width = lmargin + HTML_CLUEH (clue)->indent;
 	clue->descent = 0;
@@ -106,7 +106,7 @@ calc_size (HTMLObject *clue,
 		}
 		break;
 
-	case HTML_VALIGN_CENTER:
+	case HTML_VALIGN_MIDDLE:
 		clue->ascent = a + d;
 		for (obj = HTML_CLUE (clue)->head; obj != 0; obj = obj->next) {
 			if (obj->y != clue->ascent / 2) {
