@@ -20,11 +20,19 @@
 */
 
 #include <config.h>
+#include "gtkhtml-compat.h"
+
 #include <string.h>
 #include <stdlib.h>
-#include <unicode.h>
+
+#include <glib.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtksignal.h>
+
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include <libart_lgpl/art_rect.h>
+#include <gal/unicode/gunicode.h>
 #include <gal/widgets/e-font.h>
 
 #include "htmlentity.h"
@@ -32,6 +40,7 @@
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
 #include "htmlembedded.h"
+#include "htmlengine.h"
 #include "gtkhtml-embedded.h"
 
 static HTMLPainterClass *parent_class = NULL;
@@ -103,7 +112,8 @@ find_font (gchar *font_name, gdouble req_size, gint *font_size, GtkHTMLFontStyle
 		else if (size > req_size && (size < bigger || bigger == .0))
 			bigger = size;
 	}
-	XFreeFontNames (list);
+	if (n && list)
+		XFreeFontNames (list);
 
 	/* if not found use closest one */
 	if (!rv && (bigger || smaller)) {
@@ -121,6 +131,7 @@ find_font (gchar *font_name, gdouble req_size, gint *font_size, GtkHTMLFontStyle
 	return rv;
 }
 
+#if 0
 static void
 gtkhtml_private_split_name (gchar * c[], gchar * name)
 {
@@ -139,6 +150,7 @@ gtkhtml_private_split_name (gchar * c[], gchar * name)
 
 	c[i] = p;
 }
+#endif
 
 static gchar *
 font_name_substitute_attr (const gchar *name, gint nth, gchar *val)
@@ -162,35 +174,61 @@ font_name_substitute_attr (const gchar *name, gint nth, gchar *val)
 }
 
 static gpointer
-alloc_e_font_it (gchar *face, gdouble size, GtkHTMLFontStyle style, gchar *it)
+alloc_e_font_try (gchar *face, gdouble size, GtkHTMLFontStyle style,
+		  gchar *medium, gchar *bold, gchar *roman, gchar *italic, gboolean known_size)
 {
 	EFont *font;
 	gchar *name;
 
-
 	if (face) {
+		GdkFont *gdk_font;
 		gchar *n1, *n2, *n3, *s;
 		gint tsize;
 
-		n1 = font_name_substitute_attr (face, 3, style & GTK_HTML_FONT_STYLE_BOLD ? "bold" : "medium");
-		n2 = font_name_substitute_attr (n1,   4, style & GTK_HTML_FONT_STYLE_ITALIC ? it : "r");
+		n1 = font_name_substitute_attr (face, 3, style & GTK_HTML_FONT_STYLE_BOLD ? bold : medium);
+		n2 = font_name_substitute_attr (n1,   4, style & GTK_HTML_FONT_STYLE_ITALIC ? italic : roman);
 		n3 = font_name_substitute_attr (n2,   7, "*");
 
-		if (!find_font (n3, size, &tsize, style))
+		if (known_size) {
+			if (!find_font (n3, size, &tsize, style)) {
+				g_free (n1);
+				g_free (n2);
+				g_free (n3);
+				return NULL;
+			}
+		} else
 			tsize = size;
+
+		tsize = MAX (tsize, 1);
+
 		g_free (n1);
 		g_free (n2);
 		s    = g_strdup_printf ("%d", tsize);
 		name = font_name_substitute_attr (n3,   7, s);
 		g_free (n3);
 		g_free (s);
-	} else
-		name = g_strdup ("fixed");
 
-	font = e_font_from_gdk_name (name);
-	g_free (name);
+		/* printf ("try: %s\n", name); */
 
-	if (face) {
+		gdk_font = gdk_font_load (name);
+		if (gdk_font) {
+			font = e_font_from_gdk_name (name);
+			gdk_font_unref (gdk_font);
+		} else
+			font = NULL;
+
+		g_free (name);
+	} else {
+		GdkFont *fixed = gdk_font_load ("fixed");
+
+		if (!fixed)
+			g_error (_("Can't load fixed font."));
+		font = e_font_from_gdk_font (fixed);
+		gdk_font_unref (fixed);
+	}
+
+
+	/* if (font && face) {
 		gchar *c [14];
 
 		name = e_font_get_name (font);
@@ -202,24 +240,58 @@ alloc_e_font_it (gchar *face, gdouble size, GtkHTMLFontStyle style, gchar *it)
 			}
 			g_free (name);
 		}
-	}
+		} */
+
+	return font;
+}
+
+static EFont *
+try_font_possible_names (HTMLPainter *painter, gchar *face, gdouble size, GtkHTMLFontStyle style, gboolean known)
+{
+	EFont *font;
+
+	font = alloc_e_font_try (face, size, style, "medium", "bold", "r", "i", known);
+	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC)
+		font = alloc_e_font_try (face, size, style, "medium", "bold", "r", "o", known);
+	if (!font)
+		font = alloc_e_font_try (face, size, style, "book",   "demi", "r", "i", known);
+	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC)
+		font = alloc_e_font_try (face, size, style, "book",   "demi", "r", "o", known);
+	if (!font)
+		font = alloc_e_font_try (face, size, style, "light",   "demibold", "r", "i", known);
+	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC)
+		font = alloc_e_font_try (face, size, style, "light",   "demibold", "r", "o", known);
 
 	return font;
 }
 
 static HTMLFont *
-alloc_e_font (HTMLPainter *painter, gchar *face, gdouble size, GtkHTMLFontStyle style)
+alloc_e_font_do (HTMLPainter *painter, gchar *face, gdouble size, GtkHTMLFontStyle style)
 {
 	EFont *font;
 
-	font = alloc_e_font_it (face, size, style, "i");
-	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC)
-		font = alloc_e_font_it (face, size, style, "o");
-
+	font = try_font_possible_names (painter, face, size, style, FALSE);
 	if (!font)
-		return NULL;
+		font = try_font_possible_names (painter, face, size, style, TRUE);
 
-	return html_font_new (font, e_font_utf8_text_width (font, e_style (style), " ", 1));
+	return font ? html_font_new (font, e_font_utf8_text_width (font, e_style (style), " ", 1)) : NULL;
+}
+
+static HTMLFont *
+alloc_e_font (HTMLPainter *painter, gchar *face, gdouble size, GtkHTMLFontStyle style)
+{
+	HTMLFont *font;
+
+	font = alloc_e_font_do (painter, face, size, style);
+	if (!font && style & GTK_HTML_FONT_STYLE_BOLD)
+		font = alloc_e_font_do (painter, face, size, style & (~GTK_HTML_FONT_STYLE_BOLD));
+	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC)
+		font = alloc_e_font_do (painter, face, size, style & (~GTK_HTML_FONT_STYLE_ITALIC));
+	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC && style & GTK_HTML_FONT_STYLE_BOLD)
+		font = alloc_e_font_do (painter, face, size,
+					style & ((~GTK_HTML_FONT_STYLE_ITALIC) & (~GTK_HTML_FONT_STYLE_BOLD)));
+
+	return font;
 }
 
 static void
@@ -866,59 +938,6 @@ fill_rect (HTMLPainter *painter,
 			    width, height);
 }
 
-/* static gint
-utf8_gdk_text_width (GdkFont *font, const gchar *text, gint len)
-{
-	const gchar *p;
-	gint width, bytes;
-
-	if (!text || !font) return 0;
-
-	width = 0;
-	bytes = unicode_offset_to_index (text, len);
-	for (p = text; p && *p && p - text < bytes; p = unicode_next_utf8 (p)) {
-		unicode_char_t unival;
-		guchar c;
-		unicode_get_utf8 (p, &unival);
-		if (unival < 0) unival = '_';
-		if (unival > 255) unival = '_';
-		if (unival == 0xa0) unival = ' ';
-		c = unival;
-		width += gdk_text_width (font, &c, 1);
-	}
-
-	return width;
-}
-
-static void
-utf8_gdk_draw_text (GdkFont *font,
-		    GdkDrawable *drawable,
-		    GdkGC *gc,
-		    gint x, gint y,
-		    const gchar *text, gint len)
-{
-	const gchar *p;
-	guchar *b;
-	gint off, bytes;
-
-	if (!text || !font) return;
-
-	bytes = unicode_offset_to_index (text, len);
-	b = alloca (bytes);
-	off = 0;
-
-	for (p = text; p && *p && p - text < bytes; p = unicode_next_utf8 (p)) {
-		unicode_char_t unival;
-		unicode_get_utf8 (p, &unival);
-		if (unival < 0) unival = '_';
-		if (unival > 255) unival = '_';
-		if (unival == 0xa0) unival = ' ';
-		b [off++] = unival;
-	}
-
-	gdk_draw_text (drawable, font, gc, x, y, b, len);
-} */
-
 static EFontStyle
 e_style (GtkHTMLFontStyle style)
 {
@@ -931,15 +950,12 @@ e_style (GtkHTMLFontStyle style)
 }
 
 static void
-draw_spell_error (HTMLPainter *painter,
-		  gint x, gint y,
-		  const gchar *text,
-		  guint off, gint len)
+draw_spell_error (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len)
 {
 	HTMLGdkPainter *gdk_painter;
 	EFont *e_font;
 	GdkGCValues values;
-	guint x_off, width;
+	guint width;
 	gchar dash [2];
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
@@ -947,11 +963,9 @@ draw_spell_error (HTMLPainter *painter,
 	x -= gdk_painter->x1;
 	y -= gdk_painter->y1;
 
-	e_font = html_painter_get_font  (painter, painter->font_face, painter->font_style);
-	x_off  = e_font_utf8_text_width (e_font, e_style (painter->font_style), text,
-					 unicode_offset_to_index (text, off)) + x;
-	width  = e_font_utf8_text_width (e_font, e_style (painter->font_style), text + unicode_offset_to_index (text, off),
-					 unicode_offset_to_index (text + unicode_offset_to_index (text, off), len)) + x_off;
+	e_font  = html_painter_get_font  (painter, painter->font_face, painter->font_style);
+	width   = e_font_utf8_text_width (e_font, e_style (painter->font_style), text,
+					  g_utf8_offset_to_pointer (text, len) - text);
 
 	gdk_gc_get_values (gdk_painter->gc, &values);
 	gdk_gc_set_fill (gdk_painter->gc, GDK_OPAQUE_STIPPLED);
@@ -959,13 +973,9 @@ draw_spell_error (HTMLPainter *painter,
 	dash [1] = 2;
 	gdk_gc_set_line_attributes (gdk_painter->gc, 1, GDK_LINE_ON_OFF_DASH, values.cap_style, values.join_style);
 	gdk_gc_set_dashes (gdk_painter->gc, 2, dash, 2);
-	gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc,
-		       x_off, y, 
-		       width, y);
+	gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc, x, y, x + width, y);
 	gdk_gc_set_dashes (gdk_painter->gc, 0, dash, 2);
-	gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc,
-		       x_off, y + 1, 
-		       width, y + 1);
+	gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc, x, y + 1, x + width, y + 1);
 	gdk_gc_set_line_attributes (gdk_painter->gc, values.line_width,
 				    values.line_style, values.cap_style, values.join_style);
 }
@@ -998,7 +1008,7 @@ draw_text (HTMLPainter *painter,
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
 	if (len == -1)
-		len = unicode_strlen (text, -1);
+		len = g_utf8_strlen (text, -1);
 
 	x -= gdk_painter->x1;
 	y -= gdk_painter->y1;
@@ -1013,7 +1023,7 @@ draw_text (HTMLPainter *painter,
 			       gdk_painter->gc,
 			       x, y, 
 			       text, 
-			       unicode_offset_to_index (text, len));
+			       g_utf8_offset_to_pointer (text, len) - text);
 
 	if (painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE
 				   | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
@@ -1022,7 +1032,7 @@ draw_text (HTMLPainter *painter,
 		width = e_font_utf8_text_width (e_font,
 						e_style (painter->font_style),
 						text, 
-						unicode_offset_to_index (text, len));
+						g_utf8_offset_to_pointer (text, len) - text);
 
 		if (painter->font_style & GTK_HTML_FONT_STYLE_UNDERLINE)
 			gdk_draw_line (gdk_painter->pixmap, gdk_painter->gc, 
@@ -1095,15 +1105,14 @@ calc_text_width (HTMLPainter *painter,
 	EFont *e_font;
 	gint width;
 
-
 	gdk_painter = HTML_GDK_PAINTER (painter);
 	e_font = html_painter_get_font (painter, face, style);
 
 	width = e_font_utf8_text_width (e_font, e_style (style),
-					text, unicode_offset_to_index (text, len));
+					text, g_utf8_offset_to_pointer (text, len) - text);
 
 	/* printf ("calc_text_width text: %s len: %d\n", text, len);
-	{ gint i; printf ("["); for (i=0;i<unicode_offset_to_index (text, len); i++) printf ("%c", text [i]); printf ("] ");}
+	{ gint i; printf ("["); for (i=0;i<g_utf8_offset_to_pointer (text, len)-text; i++) printf ("%c", text [i]); printf ("] ");}
 	printf ("%d (%p)\n", width, e_font); */
 	return width;
 }
@@ -1114,6 +1123,17 @@ get_pixel_size (HTMLPainter *painter)
 	return 1;
 }
 
+static guint
+get_page_width (HTMLPainter *painter, HTMLEngine *e)
+{
+	return html_engine_get_view_width (e) + e->leftBorder + e->rightBorder;
+}
+
+static guint
+get_page_height (HTMLPainter *painter, HTMLEngine *e)
+{
+	return html_engine_get_view_height (e) + e->topBorder + e->bottomBorder;
+}
 
 static void
 init_color (GdkColor *color, gushort red, gushort green, gushort blue)
@@ -1184,6 +1204,8 @@ class_init (GtkObjectClass *object_class)
 	painter_class->draw_background = draw_background;
 	painter_class->get_pixel_size = get_pixel_size;
 	painter_class->draw_embedded = draw_embedded;
+	painter_class->get_page_width = get_page_width;
+	painter_class->get_page_height = get_page_height;
 
 	parent_class = gtk_type_class (html_painter_get_type ());
 }
