@@ -85,13 +85,25 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 	return html_text_op_cut_helper (HTML_TEXT (self), e, from, to, left, right, len, new_link);
 }
 
-static gboolean
-object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
+static int
+compare (char *str1, char *str2)
 {
-	return strcasecmp (HTML_LINK_TEXT (self)->url, HTML_LINK_TEXT (with)->url)
-		|| strcasecmp (HTML_LINK_TEXT (self)->target, HTML_LINK_TEXT (with)->target)
+	if (str1 == str2)
+		return 0;
+	
+	if (str1 && str2)
+		return strcasecmp (str1, str2);
+	else 
+		return 1;
+}
+	 
+static gboolean
+object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+{
+	return compare (HTML_LINK_TEXT (self)->url, HTML_LINK_TEXT (with)->url)
+		|| compare (HTML_LINK_TEXT (self)->target, HTML_LINK_TEXT (with)->target)
 		? FALSE
-		: (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e);
+		: (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e, left, right);
 }
 
 static const gchar *
@@ -107,11 +119,20 @@ get_target (HTMLObject *object)
 }
 
 static HTMLObject *
-remove_link (HTMLObject *self, HTMLColor *color)
+set_link (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *target)
 {
 	HTMLText *text = HTML_TEXT (self);
+	HTMLLinkText *lt = HTML_LINK_TEXT (self);
 
-	return html_text_new_with_len (text->text, text->text_len, text->font_style, color);
+	if (url) {
+		g_free (lt->url);
+		g_free (lt->target);
+		lt->url = g_strdup (url);
+		lt->target = g_strdup (target);
+		return NULL;
+	} else {	
+		return html_text_new_with_len (text->text, text->text_len, text->font_style, color);
+	}
 }
 
 static GtkHTMLFontStyle
@@ -171,7 +192,7 @@ html_link_text_class_init (HTMLLinkTextClass *klass,
 	object_class->merge = object_merge;
 	object_class->get_url = get_url;
 	object_class->get_target = get_target;
-	object_class->remove_link = remove_link;	
+	object_class->set_link = set_link;
 	object_class->save = save;
 
 	text_class->get_font_style = get_font_style;
@@ -238,13 +259,6 @@ html_link_text_new (const gchar *text,
 	return html_link_text_new_with_len (text, -1, font_style, color, url, target);
 }
 
-void
-html_link_text_set_url (HTMLLinkText *link, const gchar *url)
-{
-	g_free (link->url);
-	link->url = g_strdup (url);
-}
-
 void
 html_link_text_to_text (HTMLLinkText *link, HTMLEngine *e)
 {
