@@ -22,10 +22,12 @@
 */
 
 #include <config.h>
+#include <stdio.h>
 #include <string.h>
 
 #include "htmltextslave.h"
 #include "htmlclue.h"
+#include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
@@ -85,15 +87,87 @@ static guint
 calc_width (HTMLTextSlave *slave, HTMLPainter *painter)
 {
 	HTMLText *text = slave->owner;
-	HTMLObject *o;
+	HTMLObject *next, *prev;
 
 	if (slave->posStart == 0 && slave->posLen == text->text_len)
 		return text->word_width [text->words - 1];
 
-	o = HTML_OBJECT (slave)->next;
-	return get_words_width (text, painter, slave->start_word,
-				(o && HTML_OBJECT_TYPE (o) == HTML_TYPE_TEXTSLAVE
-				 ? HTML_TEXT_SLAVE (o)->start_word : text->words) - slave->start_word);
+	next = HTML_OBJECT (slave)->next;
+	prev = HTML_OBJECT (slave)->prev;
+	if ((prev && HTML_OBJECT_TYPE (prev) == HTML_TYPE_TEXTSLAVE
+	     && slave->start_word == HTML_TEXT_SLAVE (prev)->start_word)
+	    || (next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
+		&& slave->start_word == HTML_TEXT_SLAVE (next)->start_word)) {
+		gint line_offset = -1;
+
+		return html_painter_calc_text_width (painter, html_text_get_text (text, slave->posStart), slave->posLen,
+										  &line_offset,
+						     html_text_get_font_style (text),
+										  text->face);
+	} else
+		return get_words_width (text, painter, slave->start_word,
+					(next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
+					 ? HTML_TEXT_SLAVE (next)->start_word : text->words) - slave->start_word);
+}
+
+static gint
+get_offset_for_bounded_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *words, gint max_width)
+{
+	HTMLText *text = slave->owner;
+	gint upper = slave->posLen;
+	gint lower = 0;
+	gint len, width;
+	gint line_offset = -1;
+	gchar *sep, *str;
+	char *buffer = html_text_get_text (text, slave->posStart);
+
+	len = (lower + upper) / 2;
+	width = html_painter_calc_text_width (painter, buffer, len, &line_offset,
+					      html_text_get_font_style (text), text->face);
+	while (lower < upper) {
+		if (width > max_width)
+			upper = len - 1;
+		else
+			lower = len + 1;
+		len = (lower + upper) / 2;
+		line_offset = -1;
+		width = html_painter_calc_text_width (painter, buffer, len, &line_offset, 
+						      html_text_get_font_style (text), text->face);
+	}
+
+	if (width > max_width && len > 1)
+		len --;
+
+	*words = 0;
+	str = sep = buffer;
+
+	while ((sep = strchr (sep, ' '))) {
+		if (g_utf8_pointer_to_offset (str, sep) < len)
+			(*words) ++;
+		else
+			break;
+		sep = sep + 1;
+	}
+
+	return len;
+}
+
+static void
+slave_split_if_too_long (HTMLTextSlave *slave, HTMLPainter *painter, gint *width)
+{
+	gint x, y;
+
+	html_object_calc_abs_position (HTML_OBJECT (slave), &x, &y);
+
+	if (*width + x > MAX_WIDGET_WIDTH && slave->posLen > 1) {
+		gint words, pos;
+
+		pos = get_offset_for_bounded_width (slave, painter, &words, MAX_WIDGET_WIDTH - x);
+		if (pos > 0 && pos < slave->posLen) {
+			split (slave, pos, slave->start_word + words);
+			*width = MAX (1, calc_width (slave, painter));
+		}
+	}
 }
 
 static gboolean
@@ -127,6 +201,8 @@ calc_size (HTMLObject *self,
 	}
 
 	new_width = MAX (1, calc_width (slave, painter));
+	if (new_width > HTML_OBJECT (owner)->max_width)
+		slave_split_if_too_long (slave, painter, &new_width);
 
 	changed = FALSE;
 
@@ -174,6 +250,31 @@ debug_print (HTMLFitType rv, gchar *text, gint len)
 }
 #endif
 
+gint
+html_text_slave_get_line_offset (HTMLTextSlave *slave, gint line_offset, gint offset, HTMLPainter *p)
+{
+	HTMLObject *head = HTML_OBJECT (slave->owner)->next;
+
+	g_assert (HTML_IS_TEXT_SLAVE (head));
+
+	if (!html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (slave)->parent), p) || line_offset == -1)
+		return -1;
+
+	if (head->y + head->descent - 1 < HTML_OBJECT (slave)->y - HTML_OBJECT (slave)->ascent) {
+		HTMLObject *prev;
+		HTMLTextSlave *bol;
+
+		prev = html_object_prev (HTML_OBJECT (slave)->parent, HTML_OBJECT (slave));
+		while (prev->y + prev->descent - 1 >= HTML_OBJECT (slave)->y - HTML_OBJECT (slave)->ascent)
+			prev = html_object_prev (HTML_OBJECT (slave)->parent, HTML_OBJECT (prev));
+
+		bol = HTML_TEXT_SLAVE (prev->next);
+		return html_text_text_line_length (html_text_get_text (bol->owner, bol->posStart),
+						   0, offset - bol->posStart);
+	} else
+		return line_offset + html_text_text_line_length (slave->owner->text, line_offset, offset);
+}
+
 static gint
 get_next_nb_width (HTMLTextSlave *slave, HTMLPainter *painter)
 {
@@ -254,7 +355,7 @@ fit_line (HTMLObject *o,
 		words ++;
 		lsep   = sep;
 		sep    = strchr (lsep + (words > 1 ? 1 : 0), ' ');
-		pos    = sep ? unicode_index_to_offset (begin, sep - begin) : unicode_strlen (begin, -1);
+		pos    = sep ? g_utf8_pointer_to_offset (begin, sep) : g_utf8_strlen (begin, -1);
 		if (words + slave->start_word >= text->words)
 			break;
 	}
@@ -269,7 +370,7 @@ fit_line (HTMLObject *o,
 		else {
 			words ++;
 			sep    = strchr (sep + (words > 1 ? 0 : 1), ' ');
-			pos    = sep ? unicode_index_to_offset (begin, sep - begin) : unicode_strlen (begin, -1);
+			pos    = sep ? g_utf8_pointer_to_offset (begin, sep) : g_utf8_strlen (begin, -1);
 		}
 	}
 
@@ -319,7 +420,7 @@ get_ys (HTMLText *text, HTMLPainter *p)
 }
 
 static void
-draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
+draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty, gint line_offset)
 {
 	GList *cur = HTML_TEXT (slave->owner)->spell_errors;
 	HTMLObject *obj = HTML_OBJECT (slave);
@@ -333,12 +434,18 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 		if (ma < mi) {
 			guint off = ma - slave->posStart;
 			guint len = mi - ma;
+			gint x_off;
 
 			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLSpellErrorColor)->color);
 			/* printf ("spell error: %s\n", html_text_get_text (slave->owner, off)); */
-			html_painter_draw_spell_error (p, obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-						       html_text_get_text (HTML_TEXT (slave->owner), slave->posStart),
-						       off, len);
+			x_off = html_painter_calc_text_width (p, html_text_get_text (HTML_TEXT (slave->owner),
+										     slave->posStart), off, &line_offset,
+							      p->font_style,
+							      p->font_face);
+			html_painter_draw_spell_error (p, obj->x + tx + x_off,
+						       obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
+						       html_text_get_text (HTML_TEXT (slave->owner), slave->posStart + off),
+						       len);
 		}
 		if (se->off > slave->posStart + slave->posLen)
 			break;
@@ -352,7 +459,7 @@ draw_normal (HTMLTextSlave *self,
 	     GtkHTMLFontStyle font_style,
 	     gint x, gint y,
 	     gint width, gint height,
-	     gint tx, gint ty)
+	     gint tx, gint ty, gint line_offset)
 {
 	HTMLObject *obj;
 
@@ -365,7 +472,8 @@ draw_normal (HTMLTextSlave *self,
 	html_painter_draw_text (p,
 				obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (self->owner), p),
 				html_text_get_text (HTML_TEXT (self->owner), self->posStart),
-				self->posLen);
+				self->posLen,
+				html_text_slave_get_line_offset (self, line_offset, self->posStart, p));
 }
 
 static void
@@ -374,12 +482,12 @@ draw_highlighted (HTMLTextSlave *slave,
 		  GtkHTMLFontStyle font_style,
 		  gint x, gint y,
 		  gint width, gint height,
-		  gint tx, gint ty)
+		  gint tx, gint ty, gint line_offset)
 {
 	HTMLText *owner;
 	HTMLObject *obj;
 	guint start, end, len;
-	guint offset_width, text_width;
+	gint offset_width, text_width, lo, lo_start, lo_sel;
 	const gchar *text;
 
 	obj = HTML_OBJECT (slave);
@@ -395,11 +503,16 @@ draw_highlighted (HTMLTextSlave *slave,
 		end = slave->posStart + slave->posLen;
 	len = end - start;
 
-	offset_width = html_painter_calc_text_width (p, text + unicode_offset_to_index (text, slave->posStart),
+	lo_start = lo = html_text_slave_get_line_offset (slave, line_offset, slave->posStart, p);
+	offset_width = html_painter_calc_text_width (p, g_utf8_offset_to_pointer (text, slave->posStart),
 						     start - slave->posStart,
+						     &lo,
 						     font_style, HTML_TEXT (owner)->face);
-	text_width = html_painter_calc_text_width (p, text + unicode_offset_to_index (text, start),
-						   len, font_style, HTML_TEXT (owner)->face);
+	lo_sel = lo;
+	text_width = html_painter_calc_text_width (p, g_utf8_offset_to_pointer (text, start),
+						   len, &lo,
+						   font_style, HTML_TEXT (owner)->face);
+	/* printf ("s: %d l: %d - %d %d\n", start, len, offset_width, text_width); */
 
 	html_painter_set_font_style (p, font_style);
 	html_painter_set_font_face  (p, HTML_TEXT (owner)->face);
@@ -410,8 +523,10 @@ draw_highlighted (HTMLTextSlave *slave,
 	html_painter_fill_rect (p, obj->x + tx + offset_width, obj->y + ty - obj->ascent,
 				text_width, obj->ascent + obj->descent);
 	html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLHighlightTextColor)->color);
-	html_painter_draw_text (p, obj->x + tx + offset_width, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-				text + unicode_offset_to_index (text, start), len);
+	html_painter_draw_text (p, obj->x + tx + offset_width, 
+				obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
+				g_utf8_offset_to_pointer (text, start), len,
+				lo_sel);
 
 	/* Draw the non-highlighted part.  */
 
@@ -423,8 +538,9 @@ draw_highlighted (HTMLTextSlave *slave,
 	if (start > slave->posStart)
 		html_painter_draw_text (p,
 					obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-					text + unicode_offset_to_index (text, slave->posStart),
-					start - slave->posStart);
+					g_utf8_offset_to_pointer (text, slave->posStart),
+					start - slave->posStart,
+					lo_start);
 
 	/* 2. Draw the rightmost non-highlighted part, if any.  */
 
@@ -432,8 +548,9 @@ draw_highlighted (HTMLTextSlave *slave,
 		html_painter_draw_text (p,
 					obj->x + tx + offset_width + text_width,
 					obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-					text + unicode_offset_to_index (text, end),
-					slave->posStart + slave->posLen - end);
+					g_utf8_offset_to_pointer (text, end),
+					slave->posStart + slave->posLen - end,
+					lo);
 }
 
 static void
@@ -449,6 +566,7 @@ draw (HTMLObject *o,
 	GtkHTMLFontStyle font_style;
 	guint end;
 	ArtIRect paint;
+	gint line_offset;
 
 	html_object_calc_intersection (o, &paint, x, y, width, height);
 	if (art_irect_empty (&paint))
@@ -458,16 +576,17 @@ draw (HTMLObject *o,
 	owner = textslave->owner;
 	ownertext = HTML_TEXT (owner);
 	font_style = html_text_get_font_style (ownertext);
+	line_offset = html_text_get_line_offset (owner, p);
 
 	end = textslave->posStart + textslave->posLen;
 	if (owner->select_start + owner->select_length <= textslave->posStart
 	    || owner->select_start >= end) {
-		draw_normal (textslave, p, font_style, x, y, width, height, tx, ty);
+		draw_normal (textslave, p, font_style, x, y, width, height, tx, ty, line_offset);
 	} else {
-		draw_highlighted (textslave, p, font_style, x, y, width, height, tx, ty);
+		draw_highlighted (textslave, p, font_style, x, y, width, height, tx, ty, line_offset);
 	}
 
-	draw_spell_errors (textslave, p, tx ,ty);
+	draw_spell_errors (textslave, p, tx ,ty, line_offset);
 }
 
 static gint
@@ -500,6 +619,7 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 	GtkHTMLFontStyle font_style;
 	guint width, prev_width;
 	guint i;
+	gint line_offset, lo;
 
 	g_return_val_if_fail (slave != NULL, 0);
 
@@ -508,11 +628,13 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 
 	x -= HTML_OBJECT (slave)->x;
 
-	prev_width = 0;
+	prev_width  = 0;
+	line_offset = html_text_slave_get_line_offset (slave, html_text_get_line_offset (slave->owner, painter), 0, painter);
 	for (i = 1; i <= slave->posLen; i++) {
+		lo = line_offset;
 		width = html_painter_calc_text_width (painter,
 						      html_text_get_text (owner, slave->posStart),
-						      i, font_style, owner->face);
+						      i, &lo, font_style, owner->face);
 
 		if ((width + prev_width) / 2 >= x)
 			return i - 1;
